#include "include/raylib.h"
#include "include/raymath.h"
#include <stdlib.h>
#include <stdbool.h>
#include "lib.h"
#include "sprites.h"
#include "config.h"

#define TEXTURES_BUF_SIZE 16
#define TARGET_FPS 120
#define ANIM_SPEED 8

typedef struct {
    int frame_count;
    int anim_frames;
    bool is_attacking;
    Vector2 knight_pos;
    Rectangle knight_rect;
} GameState;

typedef struct {
    Texture2D *textures;
} Assets;

Assets Init() {
    Assets assets = { 0 };
    assets.textures = malloc(sizeof(Texture2D) * TEXTURES_BUF_SIZE);
    assets.textures[0] = LoadTexture("assets/Terrain/Ground/Tilemap_Flat.png");
    assets.textures[1] = LoadTexture("assets/Factions/Knights/Troops/Warrior/Blue/Warrior_Blue.png");
    return assets;
}

void Update(GameState *state, Camera2D cam, float dt) {
    (void)cam;
    int anim_speed = 10;
    if (state->frame_count % (TARGET_FPS/anim_speed) == 0) {
        state->anim_frames++;
        if (state->anim_frames > 5) {
            state->anim_frames = 0;
        }
        state->knight_rect.x = (float)state->anim_frames * (float)state->knight_rect.width;
    }
    float speed = 250.0f * dt;
    Vector2 input_vel = {0};
    if (IsKeyDown(KEY_RIGHT) || IsKeyDown(KEY_D)) {
        input_vel.x = 1;
    }
    if (IsKeyDown(KEY_LEFT) || IsKeyDown(KEY_A)) {
        input_vel.x = -1;
    }
    if (IsKeyDown(KEY_UP) || IsKeyDown(KEY_W)) {
        input_vel.y = -1;
    }
    if (IsKeyDown(KEY_DOWN) || IsKeyDown(KEY_S)) {
        input_vel.y = 1;
    }
    static int attacking_frames = 0;
    if (IsKeyPressed(KEY_SPACE)) {
        state->is_attacking = true;
        attacking_frames = state->anim_frames;
    }
    input_vel = Vector2Normalize(input_vel);
    input_vel = Vector2Scale(input_vel, speed);
    state->knight_pos = Vector2Add(state->knight_pos, input_vel);
    if (state->is_attacking) {
        if (state->anim_frames - attacking_frames > 5) {
            state->is_attacking = false;
            state->knight_rect.y = 0;
        } else {
            state->knight_rect.y = 192 * 2;
        }
    } else {
        if (input_vel.x != 0 || input_vel.y != 0) {
            state->knight_rect.y = 192;
        } else {
            state->knight_rect.y = 0;
        }
    }
}

void Draw(GameState *state, Assets assets, Camera2D cam, float dt) {
    (void)cam;
    (void)dt;
    ClearBackground((Color){ 100, 149, 237, 255 });

    int size = 32;
    int topx = 300;
    int topy = 32;
    for (int col = 0; col < size; col++) {
        for (int row = 0; row < size; row++) {
            int atlas_col = 0;
            int atlas_row = 0;
            if (col == size - 1) {
                atlas_col = 5;
            } else if (col > 0) {
                atlas_col = (col % 4) + 1;
            }
            if (row == size - 1) {
                atlas_row = 5;
            } else if (row > 0) {
                atlas_row = (row % 4) + 1;
            }
            Vector2 pos = {32 * col + topx, 32 * row + topy};
            Rectangle src_rect = {32 * atlas_col, 32 * atlas_row, 32, 32};
            DrawTextureRec(assets.textures[0], src_rect, pos, WHITE);
        }
    }
    DrawTextureRec(assets.textures[1], state->knight_rect, state->knight_pos, WHITE);
}

int main(void) {
    const int screen_width = 1600;
    const int screen_height = 1080;

    InitWindow(screen_width, screen_height, "raylib [core] example - basic window");

    int monitor = GetCurrentMonitor();
    int monitor_width = GetMonitorWidth(monitor);
    int monitor_height = GetMonitorHeight(monitor);
    int win_pos_x = monitor_width / 2 - screen_width / 2;
    int win_pos_y = monitor_height / 2 - screen_height / 2;
    SetWindowPosition(win_pos_x, win_pos_y);

    SetTargetFPS(TARGET_FPS);

    GameState state = {0};
    Camera2D cam = {0};

    state.knight_rect = (Rectangle) {0, 0, 1152 / 6, 1536 / 8};
    // Vector2 offset;         // Camera offset (displacement from target)
    // Vector2 target;         // Camera target (rotation and zoom origin)
    // float rotation;         // Camera rotation in degrees
    cam.zoom = 1.0f;

    Assets assets = Init();

    // const int idle
    while (!WindowShouldClose()) {
        state.frame_count++;
        float dt = GetFrameTime();

        Update(&state, cam, dt);

        BeginDrawing();
        {
            BeginMode2D(cam);
            {
                Draw(&state, assets, cam, dt);
            }
            EndMode2D();
        }
        EndDrawing();
    }

    for (int i = 0; i < TEXTURES_BUF_SIZE; i++) {
        UnloadTexture(assets.textures[0]);
    }
    CloseWindow();
    return 0;
}
